@@ -102,6 +102,7 @@ SEXP processx_write(SEXP fd, SEXP data) {
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #include <R_ext/Rdynload.h>
 #include <Rinternals.h>
@@ -125,13 +126,18 @@ static int processx__cloexec_fcntl(int fd, int set) {
   return 0;
 }
 
-SEXP processx_disable_inheritance() {
+SEXP processx_disable_inheritance(void) {
   int fd;
 
   /* Set the CLOEXEC flag on all open descriptors. Unconditionally try the
    * first 16 file descriptors. After that, bail out after the first error.
-   */
-  for (fd = 0; ; fd++) {
+   * We skip the standard streams, because R and `system()` is not prepared
+   * to not inheriting stdin and eg. an R subprocess does not even start in
+   * system(). See https://github.com/r-lib/callr/issues/236. */
+
+  int firstfd = 3;
+  if (getenv("PROCESSX_CLOEXEC_STDIO")) firstfd = 0;
+  for (fd = firstfd; ; fd++) {
     if (processx__cloexec_fcntl(fd, 1) && fd > 15) break;
   }
 
@@ -208,7 +214,6 @@ SEXP processx_set_stdout_to_file(SEXP file) {
     R_THROW_SYSTEM_ERROR("Cannot open new stdout file `%s`", c_file);
   }
   SEXP ret = processx_set_std(1, fd, 0);
-  close(fd);
   return ret;
 }
 
@@ -223,7 +228,6 @@ SEXP processx_set_stderr_to_file(SEXP file) {
     R_THROW_SYSTEM_ERROR("Cannot open new stderr file `%s`", c_file);
   }
   SEXP ret = processx_set_std(2, fd, 0);
-  close(fd);
   return ret;
 }
 
